//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, April-2009
//-----------------------------------------------------------------------------
//
// TOPsimu.cc
// C++ interface to F77 functions: fast simulation
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include "TOPsimu.h"
#include "TOPf77fun.h"

TOPsimu::TOPsimu(double T0jitter, double Bkg)
{
  float t0jitter=T0jitter; float bkg=Bkg;
  set_simu_par_(&t0jitter, &bkg);
  mtra_clear_();
}

void TOPsimu::Clear()
{
  mtra_clear_();
  data_clear_();
}

void TOPsimu::AddParticle(double X, double Y, double Z, double Tstart, double Tstop,
			  double Px, double Py, double Pz, int LUND, int REF)
{
  float x=X; float y=Y; float z=Z;  float t0=Tstart; float t1=Tstop;
  float px=Px; float py=Py; float pz=Pz;
  mtra_put_(&x, &y, &z, &t0, &t1, &px, &py, &pz, &LUND, &REF);
}

void TOPsimu::AddParticle(double X, double Y, double Z, double Tstart, 
			  double Px, double Py, double Pz, int LUND, int REF)
{
  float x=X; float y=Y; float z=Z;  float t0=Tstart; float t1=1.e9;
  float px=Px; float py=Py; float pz=Pz;
  mtra_put_(&x, &y, &z, &t0, &t1, &px, &py, &pz, &LUND, &REF);
}

void TOPsimu::AddParticle(RandomParticle & p)
{
  float x=p.X(); float y=p.Y(); float z=p.Z();  float t0=p.Tstart(); 
  float t1=p.Tstop(); float px=p.Px(); float py=p.Py(); float pz=p.Pz();
  int LUND=p.Lund(); int REF=p.Label();
  mtra_put_(&x, &y, &z, &t0, &t1, &px, &py, &pz, &LUND, &REF);
}

void TOPsimu::AddParticle(TOPparticle & p)
{
  float x=p.X(); float y=p.Y(); float z=p.Z();  float t0=p.Tof(); 
  float t1=1.e9; float px=p.Px(); float py=p.Py(); float pz=p.Pz();
  int LUND=p.Lund(); int REF=p.Label();
  mtra_put_(&x, &y, &z, &t0, &t1, &px, &py, &pz, &LUND, &REF);
}

void TOPsimu::Simulate(){ top_mcgen_();}

int TOPsimu::PartSize(){ return mtra_getnum_();}

void TOPsimu::GetNphot(int K, int & Nph, int & Mph, int & REF)
{
  K++;
  int RTRA;
  mtra_get_(&K, &Nph, &Mph, &RTRA, &REF);
}

int TOPsimu::NphExit(int Ref)
{
  if(Ref <= 0) return 0;
  int Nph, Mph, RTRA, REF;
  for(int K=1; K<=PartSize(); K++){
    mtra_get_(&K, &Nph, &Mph, &RTRA, &REF);
    if(REF==Ref) return Nph;
  }
  return 0;
}

int TOPsimu::NphDigi(int Ref)
{
  if(Ref <= 0) return 0;
  int Nph, Mph, RTRA, REF;
  for(int K=1; K<=PartSize(); K++){
    mtra_get_(&K, &Nph, &Mph, &RTRA, &REF);
    if(REF==Ref) return Mph;
  }
  return 0;
}

int TOPsimu::DataSize(){return data_getnum_();}

void TOPsimu::GetData(int K, int & QbarID, int & chID, int & TDC)
{
  K++;
  data_get_(&K, &QbarID, &chID, &TDC);
}

void TOPsimu::GetData(int K, int & QbarID, double & X, double & Y, double & Z, 
		      double & T)
{
  int chID, TDC;
  GetData(K, QbarID, chID, TDC);
  float x, y, z;
  ichxyz_(&chID, &QbarID, &x, &y, &z);
  X=x; Y=y; Z=z;
  T=tdc_to_t_(&TDC);
}

int TOPsimu::PhotonSize(){return phot_getnum_();}

void TOPsimu::GetPhotons(int K, int & QbarID, int & chID, double & X, double & Y, 
			 int & LR, double & T)
{
  K++;
  float x, y, t, e, thc, fic;
  int Nx, Ny;
  phot_get_(&K, &QbarID, &chID, &x, &y, &LR, &t, &e, &thc, &fic, &Nx, &Ny);
  X=x; Y=y; T=t; 
}

void TOPsimu::GetPhotons(int K, int & QbarID, int & chID, double & X, double & Y, 
			 int & LR, double & T, double & E)
{
  K++;
  float x, y, t, e, thc, fic;
  int Nx, Ny;
  phot_get_(&K, &QbarID, &chID, &x, &y, &LR, &t, &e, &thc, &fic, &Nx, &Ny);
  X=x; Y=y; T=t; E=e; 
}

void TOPsimu::GetPhotons(int K, int & QbarID, int & chID, double & X, double & Y, 
			 int & LR, double & T, double & E, double & ThetaC, 
			 double & PhiC)
{
  K++;
  float x, y, t, e, thc, fic;
  int Nx, Ny;
  phot_get_(&K, &QbarID, &chID, &x, &y, &LR, &t, &e, &thc, &fic, &Nx, &Ny);
  X=x; Y=y; T=t; E=e; ThetaC=thc; PhiC=fic;
}

void TOPsimu::GetPhotons(int K, int & QbarID, int & chID, double & X, double & Y, 
			 int & LR, double & T, double & E, double & ThetaC, 
			 double & PhiC, int & Nx, int & Ny)
{
  K++;
  float x, y, t, e, thc, fic;
  phot_get_(&K, &QbarID, &chID, &x, &y, &LR, &t, &e, &thc, &fic, &Nx, &Ny);
  X=x; Y=y; T=t; E=e; ThetaC=thc; PhiC=fic;
}

void TOPsimu::DumpNphot(int EvtNo)
{
  int nph, mph, ref;
  int n=0;
  for(int i=0; i<PartSize(); i++){
    GetNphot(i, nph, mph, ref);
    n+=mph;
  }
  using namespace std;
  cout << "TOPsimu::DumpNphot(): EvtNo="<< EvtNo;
  cout << "  Particles=" << PartSize(); 
  cout << "  Detected photons=" <<n<<endl;
  cout << showpoint << fixed << right;
  cout <<"                    Number of photons\n";
  cout <<"   K     Ref      QbarExit    afterDigi\n";
  for(int i=0; i<PartSize(); i++){
    GetNphot(i, nph, mph, ref);
    cout << setw(4) << i;
    cout << setw(8) << ref;
    cout << setw(12) << nph;
    cout << setw(12) << mph;
    cout << endl;
  }
  cout << "  Data size=" << DataSize() << endl;
}

void TOPsimu::DumpData(int EvtNo)
{
  int bar, ch, tdc;
  double x, y, z, t;
  using namespace std;
  cout << "TOPsimu::DumpData(): EvtNo="<< EvtNo;
  cout << "  Particles=" << PartSize(); 
  cout << "  Data size=" << DataSize() << endl;
  cout << showpoint << fixed << right;
  cout << "     K QbarID chID  TDC   X [cm]  Y [cm]  Z [cm]  T [ns]\n";
  for(int i=0; i<DataSize(); i++){
    GetData(i, bar, ch, tdc);
    cout << setw(6) << i;
    cout << setw(5) << bar;
    cout << setw(6) << ch;
    cout << setw(6) << tdc;
    GetData(i, bar, x, y, z, t);
    cout << setw(8) << setprecision(2) << x;
    cout << setw(8) << setprecision(2) << y;
    cout << setw(8) << setprecision(2) << z;
    cout << setw(8) << setprecision(2) << t;
    cout << endl;
  }
}

